""" Configuration variables used by Protean-Flask """

# Default output renderer for the app
DEFAULT_RENDERER = 'protean_flask.core.renderers.render_json'

# Default content type of the input data if none provided
DEFAULT_CONTENT_TYPE = 'application/json'

# Custom exception handler for the app
EXCEPTION_HANDLER = None
